import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import processing.serial.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class Target_Tracker_Proc3_Release_01 extends PApplet {

// ######################################################################
//
//  Target Tracker Radar File v1.0
//
//  Released:  08/12/2015
//
//  Author: TechKnowTone
//
// ######################################################################
/*
    This software is furnished "as is", without technical support, and
    with no warranty, expressed or implied, as to its usefulness for any
    purpose. Elements of this software were taken from the public domain.
    
    This code communicates with the Arduino UNO and plots received data
    on a radar style display. Reduced size for OHP screens.
  
*/

// declare libraries

Serial usbPort;

// declare variables
float angle = 0; // any angle in radians
String any = "";
int audFlag = 1; // audio ON flag, 0 = OFF, 1 = ON
int comFlag = 0; // =1 if COM serial available
String comName = ""; // serial port COM name
String data = "";
float distance = 0.0f;
int DP = 10; // degrees/pulse value
int drawFlag = 0;
PImage img;
int index = 0;
int gunFlag = 0; // = 1 if gun active is ON
int mON,mX, mY;  // mouse flag and last x,y
int originX = 400; // plot area X origin
int originY = 383; // plot area Y origin
int radFlag = 0; // = 1 if ultrasonic radar is ON
String Radius = "";
int refreshFlag = 1; // = 1 at end of sweep to redraw background
int runFlag = 0; // = 1 to run sweep
String Theta = ""; // any angle in degrees
String msgAud = "ON "; // audio button text
String msgCOM = "None"; // COM port name
String msgDP = "10 "; // degrees/pulse value
String msgGun = "OFF"; // gun button field text
String msgRad = "OFF"; // radar button field text
String msgRx = ""; // message received from Arduino
String msgSwp = "OFF"; // sweep button field text
String msgTx = ""; // message sent to Arduino
int TB = 0;  // text Blue colour component
int TG = 0;  // text Green colour component
int TR = 0;  // text Red colour component
float x, y;  // co-ordinates of sweep dot

public void setup() {
    // screen size must match background bitmap
  img = loadImage("Background.png");
  mON = 0;
  drawRadarScrn();
  getCOMPort();
  // allow UNO time to boot up
  delay(1500);
  println("Ready!");
}

public void draw() {
  // main loop. Comes here about 60 times a second
  if (mON > 0) {drawMouseXY();}
  if (drawFlag > 0) {
    if (runFlag == 0) {drawRadarScrn();}
    drawRxMsg();
    fill(255,255,30); ellipse(x, y, 10, 10); drawFlag = 0;
  }
  if (refreshFlag > 0) {drawRadarScrn(); refreshFlag = 0;}
}

// ----------------------------------------------------------------------

public void drawCOMMsg() {
  // put text in COM value field
  if (comFlag < 1) {TR = 255;}
  drawTxtField(msgCOM, 92, 494); TR = 0;
}

// ----------------------------------------------------------------------

public void drawDPMsg() {
  // put text in D/P value field
  drawTxtField(msgDP, 630, 429);
}

// ----------------------------------------------------------------------

public void drawGunMsg() {
  // put text in weapon ON/OFF button field
  drawTxtField(msgGun, 457, 461);
}

// ----------------------------------------------------------------------

public void drawMouseXY() {
  // draws the value of mouse x,y top left
  if ((mouseX != mX) || (mouseY != mY)) {
    fill(200,200,200); rect(10,8,50,30);
    fill(0,0,0); text("X=" + mouseX, 12, 20); text("Y=" + mouseY, 12, 34);
    mX = mouseX; mY = mouseY;
  }
}

// ----------------------------------------------------------------------

public void drawRadarScrn() {
  // draw the radar style display
  background(img); // define background RGB colour
  drawSwpMsg();
  drawRadMsg(); drawDPMsg();
  drawGunMsg();
  drawCOMMsg(); drawRxMsg(); drawTxMsg();
}

// ----------------------------------------------------------------------

public void drawRadMsg() {
  // put text in radar ON/OFF button field
  drawTxtField(msgRad, 457, 429);
}

// ----------------------------------------------------------------------

public void drawRxMsg() {
  // put text in the Rx field
  drawTxtField("                            ", 244, 680);
  drawTxtField(msgRx, 248, 494);
}

// ----------------------------------------------------------------------

public void drawSwpMsg() {
  // put text in sweep ON/OFF button field
  drawTxtField(msgSwp, 457, 397);
}

// ----------------------------------------------------------------------

public void drawTxMsg() {
  // put text in Tx field
  drawTxtField("                            ", 540, 680);
  drawTxtField(msgTx, 528, 494);
}

// ----------------------------------------------------------------------

public void drawTxtField(String zM,int zX, int zY) {
  stroke(242,242,242); fill(242,242,242); 
  rect(zX, zY, 8+textWidth(zM), 14);
  fill(TR,TG,TB); text(zM, zX+4, zY+12);
}

// ----------------------------------------------------------------------

public void exit() {
  // program is closing so reset Arduino...
  usbPortWrite("XX.");
  delay(20); // allow 20ms for data to be sent
  super.exit();
}

// ----------------------------------------------------------------------

public void getCOMPort() {
  // initialise serial comms if a port exists
  if (comFlag > 0) {
    // stops an existing connection if already made
    usbPort.stop();
    comFlag = 0; msgCOM = "----"; delay(100);
  }
  if (Serial.list().length > 0) {
    // test available serial port
    comName = Serial.list()[0];
    println(comName);
    usbPort = new Serial(this, comName, 19200);
    usbPort.bufferUntil('.');
    comFlag = 1; msgCOM = comName;
  } else {
    // no serial port available
    comFlag = 0; msgCOM = "----";
  } refreshFlag = 1;
}

// ----------------------------------------------------------------------

public void mousePressed() {
  // mouse has been clicked on window
  mX = mouseX; mY = mouseY;
  if ((mX < 60) && (mY < 40)) {
    // toggle mouse X,Y display ON/OFF
    if (mON < 1) {mON = 1;} else {mON = 0;} refreshFlag = 1;
  }
  if ((mX > 239) && (mX < 522)) {
    if ((mY > 387) && (mY < 421)) {toggleRadSweep();}
    if ((mY > 419) && (mY < 452)) {toggleRadOnOff();}
    if ((mY > 451) && (mY < 484)) {toggleGunOnOff();}
  }
  if ((mX > 521) && (mX < 6675)) {
    if ((mY > 387) && (mY < 421)) {sendRadCntr();}
    if ((mY > 451) && (mY < 484)) {sendGunCntr();}
  }
  if ((mX > 582) && (mX < 614)) {
    if ((mY > 419) && (mY < 436)) {sendDPInc(1);}
    if ((mY > 435) && (mY < 453)) {sendDPInc(-1);}
  }
  if ((mX > 718) && (mX < 778) && (mY > 22) && (mY < 84)) {refreshFlag = 1;}
  if ((mX > 29) && (mX < 770) && (mY > 15) && (mY < originY)) {radarClick();}
  if ((mX > 85) && (mX < 172) && (mY > 674) && (mY < 700)) {getCOMPort();}
  if ((mY > 419) && (mY < 453)) {
    if ((mX > 37) && (mX < 94)) {sendMode(1);}
    if ((mX > 92) && (mX < 148)) {sendMode(2);}
    if ((mX > 146) && (mX < 202)) {sendMode(3);}
  }
}

// ----------------------------------------------------------------------

public void mouseDragged() {
  // mouse is been dragged on window with button pressed
  mX = mouseX; mY = mouseY;
  if ((mX > 29) && (mX < 770) && (mY > 29) && (mY < 400)) {radarClick();}
}

// ----------------------------------------------------------------------

public void radarClick() {
  // mouse has bee pressed in radar region
  // get length of line
  float zXL = abs(originX - mX - 1);
  float zYL = abs(originY - mY - 1);
  float zRL = sqrt(sq(zXL) + sq(zYL));
  if (zRL <= 370.0f) {
    if (zRL >= 343.0f) {
      // clicked outer-radius are so calculate angle
      if (runFlag > 0) {runFlag = 0; msgSwp = "OFF"; drawSwpMsg();}
      angle = atan(zXL/zYL);
      float zTheta = 180 * angle / PI;
      if (mX <= (originX - 1)) {
        msgTx = "RM" + str(PApplet.parseInt(90 - zTheta)) + ".";
      } else {
        msgTx = "RM" + str(PApplet.parseInt(90 + zTheta)) + ".";
      } drawTxMsg(); usbPortWrite(msgTx);
    } else {
      //msgTx = str(zRL); drawTxMsg();
    }
  }
}

// ----------------------------------------------------------------------

public void sendDPInc(int zInc) {
  // change the Degrees/Pulse value
  DP = DP + zInc;
  if (DP > 10) {DP = 10;}
  if (DP < 1) {DP = 1;}
  msgTx = "DP" + str(DP) + "."; usbPortWrite(msgTx); drawTxMsg();
  msgDP = str(DP) + " "; drawDPMsg();
}

// ----------------------------------------------------------------------

public void sendGunCntr() {
  // centralise gun
  msgTx = "GC."; usbPortWrite(msgTx); drawTxMsg();
}

// ----------------------------------------------------------------------

public void sendMode(int zMode) {
  // mode button pressed so send corresponding message
  switch (zMode) {
    case 1: msgTx = "M1.";
      runFlag = 0; msgSwp = "OFF";
      radFlag = 0; msgRad = "OFF";
      gunFlag = 0; msgGun = "OFF"; break;
    case 2: msgTx = "M2.";
      runFlag = 1; msgSwp = " ON";
      radFlag = 1; msgRad = " ON";
      gunFlag = 0; msgGun = "OFF"; break;
    case 3: msgTx = "M3.";
      runFlag = 1; msgSwp = " ON";
      radFlag = 1; msgRad = " ON";
      gunFlag = 1; msgGun = " ON"; break;
  }
  usbPortWrite(msgTx); drawTxMsg(); refreshFlag = 1;
}

// ----------------------------------------------------------------------

public void sendRadCntr() {
  // centralise radar
  msgTx = "RC."; usbPortWrite(msgTx); drawTxMsg();
  runFlag = 0; msgSwp = "OFF"; drawSwpMsg();
}

// ----------------------------------------------------------------------

public void serialEvent(Serial usbPort) {
  // respond to a serial data received event
  data = usbPort.readStringUntil('.');
  String test = data.substring(2,3);
  if (test.equals(">") == true) {
    //not a command so assume it's data
    data = data.substring(3, data.length()-1);
    msgRx = ">" + data;
    index = data.indexOf(",");
    //println(data);
    Radius = data.substring(0, index);
    distance = PApplet.parseFloat(Radius)/12.4f; // set correct scale for graph
    if (distance > 0) {
      Theta = data.substring (index+1 , data.length());
      // Draw received data
      //println("Distance=" + distance);
      angle = (PApplet.parseFloat(Theta) /180) * PI; 
      //println("Angle=" + angle);
      x = originX - (distance * cos(angle));
      y = originY + (-1 * distance * sin(angle));
      drawFlag = 1;
      //println("x=" + x + ", y=" + y);
    }
    else if (distance < 0) {refreshFlag = 1;}
  }
  else {data = "";}
}

// ----------------------------------------------------------------------

public void toggleGunOnOff() {
  // toggle gun active ON and OFF
  if (gunFlag > 0) {
    msgTx = "GA0."; gunFlag = 0; msgGun = "OFF";
  }
  else {
    msgTx = "GA1."; gunFlag = 1; msgGun = " ON";
  }
  usbPortWrite(msgTx); drawTxMsg();
  drawGunMsg();
}

// ----------------------------------------------------------------------

public void toggleRadOnOff() {
  // toggles the radar ultrasonics ON/OFF
  if (radFlag > 0) {
    msgTx = "U0."; radFlag = 0; msgRad = "OFF";
  }
  else {
    msgTx = "U1."; radFlag = 1; msgRad = " ON";
  }
  usbPortWrite(msgTx); drawTxMsg();
  drawRadMsg();
}

// ----------------------------------------------------------------------

public void toggleRadSweep() {
  // toggle radar sweep ON/OFF
  if (runFlag > 0) {
    msgTx = "RS0."; runFlag = 0; msgSwp = "OFF";
  }
  else {
    msgTx = "RS1."; runFlag = 1; msgSwp = " ON";
  }
  usbPortWrite(msgTx); drawTxMsg();
  drawSwpMsg();
}

// ----------------------------------------------------------------------

public void usbPortWrite(String zmsg) {
  // called to send text over the serial port
  if (comFlag > 0) {usbPort.write(zmsg);}
}
  public void settings() {  size(800,528); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "Target_Tracker_Proc3_Release_01" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
